@@ -1,6 +1,6 @@
 ## RcppArmadillo.package.skeleton.R: makes a skeleton for a package that wants to use RcppArmadillo
 ##
-## Copyright (C)  2010 - 2019  Dirk Eddelbuettel, Romain Francois and Douglas Bates
+## Copyright (C)  2010 - 2021  Dirk Eddelbuettel, Romain Francois and Douglas Bates
 ##
 ## This file is part of RcppArmadillo.
 ##
@@ -23,7 +23,7 @@ RcppArmadillo.package.skeleton <- function(name="anRpackage", list=character(),
                                            code_files=character(),
                                            example_code=TRUE) {
 
-    env <- parent.frame(1)
+    env <- parent.frame(1)              # #nocov start
 
     if (! length(list)) {
         fake <- TRUE
@@ -78,7 +78,12 @@ RcppArmadillo.package.skeleton <- function(name="anRpackage", list=character(),
     ## add a useDynLib to NAMESPACE,
     NAMESPACE <- file.path( root, "NAMESPACE")
     lines <- readLines( NAMESPACE )
-    if (! grepl("useDynLib", lines)) {
+    ## remove *.fake.fun and set exportPattern (will also be in Rcpp 1.0.5)
+    lines <- lines[!grepl("^export.*fake\\.fun", lines)]
+    if (!any(grepl("^exportPattern", lines))) {
+        lines <- c(lines, "exportPattern(\"^[[:alpha:]]+\")")
+    }
+    if (!any(grepl("useDynLib", lines))) {
         lines <- c(sprintf("useDynLib(%s, .registration=TRUE)", name),
                    "importFrom(Rcpp, evalCpp)",        ## ensures Rcpp instantiation
                    lines)
@@ -124,5 +129,5 @@ RcppArmadillo.package.skeleton <- function(name="anRpackage", list=character(),
         unlink(file.path(root, "man", "Rcpp.fake.fun.Rd"))
     }
 
-    invisible(NULL)
+    invisible(NULL) 						# #nocov end
 }
