@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -86,30 +88,47 @@ class field
   template<typename oT2>
   inline void copy_size(const field<oT2>& x);
   
-  arma_inline       oT& operator[](const uword i);
-  arma_inline const oT& operator[](const uword i) const;
+  arma_warn_unused arma_inline       oT& operator[](const uword i);
+  arma_warn_unused arma_inline const oT& operator[](const uword i) const;
   
-  arma_inline       oT&         at(const uword i);
-  arma_inline const oT&         at(const uword i) const;
+  arma_warn_unused arma_inline       oT&         at(const uword i);
+  arma_warn_unused arma_inline const oT&         at(const uword i) const;
   
-  arma_inline       oT& operator()(const uword i);
-  arma_inline const oT& operator()(const uword i) const;
+  arma_warn_unused arma_inline       oT& operator()(const uword i);
+  arma_warn_unused arma_inline const oT& operator()(const uword i) const;
   
-  arma_inline       oT&         at(const uword row, const uword col);
-  arma_inline const oT&         at(const uword row, const uword col) const;
+  #if defined(__cpp_multidimensional_subscript)
+  arma_warn_unused arma_inline       oT& operator[](const uword row, const uword col);
+  arma_warn_unused arma_inline const oT& operator[](const uword row, const uword col) const;
+  #endif
+  
+  arma_warn_unused arma_inline       oT&         at(const uword row, const uword col);
+  arma_warn_unused arma_inline const oT&         at(const uword row, const uword col) const;
+  
+  #if defined(__cpp_multidimensional_subscript)
+  arma_warn_unused arma_inline       oT& operator[](const uword row, const uword col, const uword slice);
+  arma_warn_unused arma_inline const oT& operator[](const uword row, const uword col, const uword slice) const;
+  #endif
+  
+  arma_warn_unused arma_inline       oT&         at(const uword row, const uword col, const uword slice);
+  arma_warn_unused arma_inline const oT&         at(const uword row, const uword col, const uword slice) const;
+  
+  arma_warn_unused arma_inline       oT& operator()(const uword row, const uword col);
+  arma_warn_unused arma_inline const oT& operator()(const uword row, const uword col) const;
 
-  arma_inline       oT&         at(const uword row, const uword col, const uword slice);
-  arma_inline const oT&         at(const uword row, const uword col, const uword slice) const;
-  
-  arma_inline       oT& operator()(const uword row, const uword col);
-  arma_inline const oT& operator()(const uword row, const uword col) const;
-
-  arma_inline       oT& operator()(const uword row, const uword col, const uword slice);
-  arma_inline const oT& operator()(const uword row, const uword col, const uword slice) const;
+  arma_warn_unused arma_inline       oT& operator()(const uword row, const uword col, const uword slice);
+  arma_warn_unused arma_inline const oT& operator()(const uword row, const uword col, const uword slice) const;
   
   
-  arma_cold inline field_injector<field> operator<<(const oT& val);
-  arma_cold inline field_injector<field> operator<<(const injector_end_of_row<>& x);
+  arma_warn_unused arma_inline       oT& front();
+  arma_warn_unused arma_inline const oT& front() const;
+  
+  arma_warn_unused arma_inline       oT& back();
+  arma_warn_unused arma_inline const oT& back() const;
+  
+  
+  arma_frown("use braced initialiser list instead") inline field_injector<field> operator<<(const oT& val);
+  arma_frown("use braced initialiser list instead") inline field_injector<field> operator<<(const injector_end_of_row<>& x);
   
   
   inline       subview_field<oT> row(const uword row_num);
@@ -175,34 +194,34 @@ class field
   arma_inline bool is_empty() const;
   
   
-  arma_inline arma_warn_unused bool in_range(const uword i) const;
-  arma_inline arma_warn_unused bool in_range(const span& x) const;
+  arma_warn_unused arma_inline bool in_range(const uword i) const;
+  arma_warn_unused arma_inline bool in_range(const span& x) const;
   
-  arma_inline arma_warn_unused bool in_range(const uword   in_row, const uword   in_col) const;
-  arma_inline arma_warn_unused bool in_range(const span& row_span, const uword   in_col) const;
-  arma_inline arma_warn_unused bool in_range(const uword   in_row, const span& col_span) const;
-  arma_inline arma_warn_unused bool in_range(const span& row_span, const span& col_span) const;
+  arma_warn_unused arma_inline bool in_range(const uword   in_row, const uword   in_col) const;
+  arma_warn_unused arma_inline bool in_range(const span& row_span, const uword   in_col) const;
+  arma_warn_unused arma_inline bool in_range(const uword   in_row, const span& col_span) const;
+  arma_warn_unused arma_inline bool in_range(const span& row_span, const span& col_span) const;
   
-  arma_inline arma_warn_unused bool in_range(const uword   in_row, const uword   in_col, const SizeMat& s) const;
+  arma_warn_unused arma_inline bool in_range(const uword   in_row, const uword   in_col, const SizeMat& s) const;
   
-  arma_inline arma_warn_unused bool in_range(const uword   in_row, const uword   in_col, const uword   in_slice) const;
-  arma_inline arma_warn_unused bool in_range(const span& row_span, const span& col_span, const span& slice_span) const;
+  arma_warn_unused arma_inline bool in_range(const uword   in_row, const uword   in_col, const uword   in_slice) const;
+  arma_warn_unused arma_inline bool in_range(const span& row_span, const span& col_span, const span& slice_span) const;
   
-  arma_inline arma_warn_unused bool in_range(const uword   in_row, const uword in_col, const uword in_slice, const SizeCube& s) const;
+  arma_warn_unused arma_inline bool in_range(const uword   in_row, const uword in_col, const uword in_slice, const SizeCube& s) const;
   
   
-  inline arma_cold bool save(const std::string   name, const file_type type = arma_binary, const bool print_status = true) const;
-  inline arma_cold bool save(      std::ostream& os,   const file_type type = arma_binary, const bool print_status = true) const;
+  arma_cold inline bool save(const std::string   name, const file_type type = arma_binary) const;
+  arma_cold inline bool save(      std::ostream& os,   const file_type type = arma_binary) const;
   
-  inline arma_cold bool load(const std::string   name, const file_type type = auto_detect, const bool print_status = true);
-  inline arma_cold bool load(      std::istream& is,   const file_type type = auto_detect, const bool print_status = true);
+  arma_cold inline bool load(const std::string   name, const file_type type = auto_detect);
+  arma_cold inline bool load(      std::istream& is,   const file_type type = auto_detect);
   
   
-  inline arma_cold bool quiet_save(const std::string   name, const file_type type = arma_binary) const;
-  inline arma_cold bool quiet_save(      std::ostream& os,   const file_type type = arma_binary) const;
+  arma_deprecated inline bool quiet_save(const std::string   name, const file_type type = arma_binary) const;
+  arma_deprecated inline bool quiet_save(      std::ostream& os,   const file_type type = arma_binary) const;
   
-  inline arma_cold bool quiet_load(const std::string   name, const file_type type = auto_detect);
-  inline arma_cold bool quiet_load(      std::istream& is,   const file_type type = auto_detect);
+  arma_deprecated inline bool quiet_load(const std::string   name, const file_type type = auto_detect);
+  arma_deprecated inline bool quiet_load(      std::istream& is,   const file_type type = auto_detect);
   
   
   // for container-like functionality
@@ -283,7 +302,7 @@ class field
   
   public:
   
-  #ifdef ARMA_EXTRA_FIELD_PROTO
+  #if defined(ARMA_EXTRA_FIELD_PROTO)
     #include ARMA_INCFILE_WRAP(ARMA_EXTRA_FIELD_PROTO)
   #endif
   };
