@@ -1052,7 +1052,6 @@ gmm_diag<eT>::internal_gen_boundaries(const uword N) const
 
 
 template<typename eT>
-arma_hot
 inline
 eT
 gmm_diag<eT>::internal_scalar_log_p(const eT* x) const
@@ -1085,7 +1084,6 @@ gmm_diag<eT>::internal_scalar_log_p(const eT* x) const
 
 
 template<typename eT>
-arma_hot
 inline
 eT
 gmm_diag<eT>::internal_scalar_log_p(const eT* x, const uword g) const
@@ -2159,6 +2157,10 @@ gmm_diag<eT>::km_iterate(const Mat<eT>& X, const uword max_iter, const bool verb
       }
     #else
       {
+      acc_hefts.zeros();
+      acc_means.zeros();
+      last_indx.zeros();
+      
       uword* acc_hefts_mem = acc_hefts.memptr();
       uword* last_indx_mem = last_indx.memptr();
       
