@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -20,7 +22,6 @@
 
 
 template<typename eT>
-arma_cold
 inline
 mat_injector_row<eT>::mat_injector_row()
   : n_cols(0)
@@ -33,7 +34,6 @@ mat_injector_row<eT>::mat_injector_row()
 
 
 template<typename eT>
-arma_cold
 inline
 void
 mat_injector_row<eT>::insert(const eT val) const
@@ -68,7 +68,6 @@ mat_injector_row<eT>::insert(const eT val) const
 
 
 template<typename T1>
-arma_cold
 inline
 mat_injector<T1>::mat_injector(T1& in_X, const typename mat_injector<T1>::elem_type val)
   : X(in_X)
@@ -96,7 +95,6 @@ mat_injector<T1>::mat_injector(T1& in_X, const typename mat_injector<T1>::elem_t
 
 
 template<typename T1>
-arma_cold
 inline
 mat_injector<T1>::mat_injector(T1& in_X, const injector_end_of_row<>& x)
   : X(in_X)
@@ -125,7 +123,6 @@ mat_injector<T1>::mat_injector(T1& in_X, const injector_end_of_row<>& x)
 
 
 template<typename T1>
-arma_cold
 inline
 mat_injector<T1>::~mat_injector()
   {
@@ -224,7 +221,6 @@ mat_injector<T1>::~mat_injector()
 
 
 template<typename T1>
-arma_cold
 inline
 void
 mat_injector<T1>::insert(const typename mat_injector<T1>::elem_type val) const
@@ -242,7 +238,6 @@ mat_injector<T1>::insert(const typename mat_injector<T1>::elem_type val) const
 
 
 template<typename T1>
-arma_cold
 inline
 void
 mat_injector<T1>::end_of_row() const
@@ -271,7 +266,7 @@ mat_injector<T1>::end_of_row() const
 
 
 template<typename T1>
-arma_cold
+inline
 const mat_injector<T1>&
 operator<<(const mat_injector<T1>& ref, const typename mat_injector<T1>::elem_type val)
   {
@@ -285,7 +280,7 @@ operator<<(const mat_injector<T1>& ref, const typename mat_injector<T1>::elem_ty
 
 
 template<typename T1>
-arma_cold
+inline
 const mat_injector<T1>&
 operator<<(const mat_injector<T1>& ref, const injector_end_of_row<>& x)
   {
@@ -300,7 +295,7 @@ operator<<(const mat_injector<T1>& ref, const injector_end_of_row<>& x)
 
 
 //// using a mixture of operator << and , doesn't work yet
-//// e.g. A << 1, 2, 3 << endr
+//// eg. A << 1, 2, 3 << endr
 //// in the above "3 << endr" requires special handling.
 //// similarly, special handling is necessary for "endr << 3"
 //// 
@@ -341,7 +336,6 @@ operator<<(const mat_injector<T1>& ref, const injector_end_of_row<>& x)
 
 
 template<typename oT>
-arma_cold
 inline
 field_injector_row<oT>::field_injector_row()
   : n_cols(0)
@@ -359,7 +353,6 @@ field_injector_row<oT>::field_injector_row()
 
 
 template<typename oT>
-arma_cold
 inline
 field_injector_row<oT>::~field_injector_row()
   {
@@ -372,7 +365,6 @@ field_injector_row<oT>::~field_injector_row()
 
 
 template<typename oT>
-arma_cold
 inline
 void
 field_injector_row<oT>::insert(const oT& val) const
@@ -411,7 +403,6 @@ field_injector_row<oT>::insert(const oT& val) const
 
 
 template<typename T1>
-arma_cold
 inline
 field_injector<T1>::field_injector(T1& in_X, const typename field_injector<T1>::object_type& val)
   : X(in_X)
@@ -439,7 +430,6 @@ field_injector<T1>::field_injector(T1& in_X, const typename field_injector<T1>::
 
 
 template<typename T1>
-arma_cold
 inline
 field_injector<T1>::field_injector(T1& in_X, const injector_end_of_row<>& x)
   : X(in_X)
@@ -468,7 +458,6 @@ field_injector<T1>::field_injector(T1& in_X, const injector_end_of_row<>& x)
 
 
 template<typename T1>
-arma_cold
 inline
 field_injector<T1>::~field_injector()
   {
@@ -526,7 +515,6 @@ field_injector<T1>::~field_injector()
 
 
 template<typename T1>
-arma_cold
 inline
 void
 field_injector<T1>::insert(const typename field_injector<T1>::object_type& val) const
@@ -544,7 +532,6 @@ field_injector<T1>::insert(const typename field_injector<T1>::object_type& val)
 
 
 template<typename T1>
-arma_cold
 inline
 void
 field_injector<T1>::end_of_row() const
@@ -576,7 +563,7 @@ field_injector<T1>::end_of_row() const
 
 
 template<typename T1>
-arma_cold
+inline
 const field_injector<T1>&
 operator<<(const field_injector<T1>& ref, const typename field_injector<T1>::object_type& val)
   {
@@ -590,7 +577,7 @@ operator<<(const field_injector<T1>& ref, const typename field_injector<T1>::obj
 
 
 template<typename T1>
-arma_cold
+inline
 const field_injector<T1>&
 operator<<(const field_injector<T1>& ref, const injector_end_of_row<>& x)
   {
