@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -41,9 +43,37 @@ mtSpOp<out_eT, T1, op_type>::mtSpOp(const T1& in_m, const uword in_aux_uword_a,
 
 
 
+template<typename out_eT, typename T1, typename op_type>
+inline
+mtSpOp<out_eT, T1, op_type>::mtSpOp(const char junk, const T1& in_m, const out_eT in_aux)
+  : m(in_m)
+  , aux_out_eT(in_aux)
+  {
+  arma_ignore(junk);
+  
+  arma_extra_debug_sigprint();
+  }
+
+
+
 template<typename out_eT, typename T1, typename op_type>
 inline
 mtSpOp<out_eT, T1, op_type>::~mtSpOp()
   {
   arma_extra_debug_sigprint();
   }
+
+
+
+template<typename out_eT, typename T1, typename op_type>
+template<typename eT2>
+arma_inline
+bool
+mtSpOp<out_eT, T1, op_type>::is_alias(const SpMat<eT2>& X) const
+  {
+  return (void_ptr(&X) == void_ptr(&m));
+  }
+
+
+
+//! @}
